import json
import datetime
from flask import jsonify, Response
from concurrent.futures import ThreadPoolExecutor
from json.decoder import JSONDecodeError
from flask import Blueprint, current_app
from brian_dashboard_manager.routes import common
from brian_dashboard_manager.grafana.provision import provision_maybe
from brian_dashboard_manager import CONFIG_KEY
from brian_dashboard_manager.config import STATE_PATH


routes = Blueprint("update", __name__)

UPDATE_RESPONSE_SCHEMA = {
    '$schema': 'http://json-schema.org/draft-07/schema#',
    'type': 'object',
    'properties': {
        'message': {
            'type': 'string'
        }
    }
}


@routes.after_request
def after_request(resp):
    return common.after_request(resp)


def should_provision():
    """
    Check if we should provision by checking the state file.
    Multiple workers can call this function at the same time,
    so we need to make sure we don't provision twice while
    the first provisioning is still running.

    :return: tuple of (bool, datetime) representing if we can provision
        and the timestamp of the last provisioning, respectively.
    """

    try:
        with open(STATE_PATH, 'r+') as f:
            try:
                state = json.load(f)
            except JSONDecodeError:
                state = {}

            provisioning = state.get('provisioning', False)
            timestamp = datetime.datetime.fromtimestamp(
                state.get('timestamp', 1))

            now = datetime.datetime.now()
            if provisioning and (now - timestamp).total_seconds() > 3600:
                # if we stay in provisioning state
                # for over an hour, we probably restarted
                # and the state file is out of sync.
                provisioning = False

            can_provision = not provisioning
            return can_provision, timestamp
    except FileNotFoundError:
        with open(STATE_PATH, 'w') as f:
            return True, None


@routes.route('/', methods=['GET'])
def update():
    """
    This resource is used to trigger the provisioning to Grafana.

    It responds to the request immediately after starting
    the provisioning process.

    The response will be formatted according to the following schema:

    .. asjson::
       brian_dashboard_manager.routes.update.UPDATE_RESPONSE_SCHEMA

    :return: json
    """
    should, timestamp = should_provision()
    if should:
        executor = ThreadPoolExecutor(max_workers=1)
        executor.submit(provision_maybe, current_app.config[CONFIG_KEY])
        return jsonify({'data': {'message': 'Provisioning dashboards!'}})
    else:
        message = f'Provision already in progress since {timestamp}'
        return Response(message, status=503)
